@@ -1,4 +1,5 @@
-### $Id: xport.R,v 1.2 2001/06/09 14:40:20 hornik Exp $
+### This file is part of the 'foreign' package for R.
+
 ###
 ###             Read SAS xport format libraries
 ###
@@ -9,26 +10,23 @@
 ### It is made available under the terms of the GNU General Public
 ### License, version 2, or at your option, any later version,
 ### incorporated herein by reference.
-### 
+###
 ### This program is distributed in the hope that it will be
 ### useful, but WITHOUT ANY WARRANTY; without even the implied
 ### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 ### PURPOSE.  See the GNU General Public License for more
 ### details.
-### 
+###
 ### You should have received a copy of the GNU General Public
-### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### License along with this program; if not, a copy is available at
+### http://www.r-project.org/Licenses/
+
+lookup.xport <- function(file) .Call(xport_info, file)
 
-lookup.xport <- function(file) {
-    .Call("xport_info", file, PACKAGE = "foreign")
-}
 
 read.xport <- function(file) {
     data.info <- lookup.xport(file)
-    ans <- .Call("xport_read", file, data.info, PACKAGE = "foreign")
-    if (length(ans) == 1)
-        as.data.frame(ans[[1]])
+    ans <- .Call(xport_read, file, data.info)
+    if (length(ans) == 1L) as.data.frame(ans[[1L]])
     else lapply(ans, as.data.frame)
 }
